USE [igops]
GO

DROP VIEW IF EXISTS reassignments_v;
DROP VIEW IF EXISTS review_item_reason_v;
DROP VIEW IF EXISTS review_parameter_v;
GO

CREATE VIEW reassignments_v AS
  SELECT
    history.review_instance_id,
    history.review_item_id,
    history.review_order,
    review.name AS review_name,
    review.status AS review_status,
    review.start_date,
    review.end_date,
      CASE
          WHEN item.item_type = 'PERMISSION' THEN perm.name
          WHEN item.item_type = 'ACCOUNT' THEN acct.name
          WHEN item.item_type = 'ROLE' THEN role.name
          WHEN item.item_type = 'BUSINESS_ROLE' THEN brole.name
      END AS review_item,
    item.item_type,
    history.reassigned_by_user_unique_id,
    history.reassigned_by,
    history.reassigned_from,
    history.reassigned_to,
    reason.id AS reason_id,
    reason.name AS reassigned_reasons,
    history.reviewer_comment,
    history.reassignment_date
  FROM ( SELECT
      ria_1.id,
      ria_1.review_item_id,
      ria_1.review_order,
      ria_1.reason_id,
      ria_1.reviewer_comment,
      ria_1.action,
      ria_1.reassigned_by_user_unique_id,
        CASE
            WHEN reassign_from_multi.users IS NOT NULL AND reassign_from_multi.grps IS NOT NULL THEN concat(reassign_from_multi.users, concat(' | ', reassign_from_multi.grps))
            ELSE COALESCE(reassign_from_multi.grps, reassign_from_multi.users, reassign_from.display_name)
        END AS reassigned_from,
      reassign_by.display_name AS reassigned_by,
      reassign_to.display_name AS reassigned_to,
      ria_1.review_instance_id,
        CASE
            WHEN ria_1.assigned IS NULL AND ria_1.action = 'REASSIGNED' THEN null_date.assigned
            ELSE ria_1.assigned
        END AS reassignment_date
    FROM review_item_action ria_1
      JOIN reviewer_instance ri ON ri.id = ria_1.reviewer_instance_id
      JOIN suser reassign_from ON ri.single_reviewer_unique_id = reassign_from.unique_user_id AND reassign_from.effective = 1
      JOIN suser reassign_by ON reassign_by.unique_user_id = ria_1.reassigned_by_user_unique_id AND reassign_by.effective = 1
      JOIN suser reassign_to ON reassign_to.unique_user_id = ria_1.reassigned_to_user_unique_id AND reassign_to.effective = 1
      LEFT JOIN review_item_action null_date ON null_date.review_item_id = ria_1.review_item_id AND null_date.review_order = ria_1.review_order AND null_date.action = 'PENDING'
      LEFT JOIN ( SELECT 
          ria_1_2.id,
          (STUFF((SELECT ' | ' + reassign_from_users_1.display_name 
              FROM review_item_action ria_1_1
                JOIN reviewer_set_instance rsi_1 ON ria_1_1.reviewer_instance_id = rsi_1.reviewer_instance_id
                LEFT JOIN suser reassign_from_users_1 ON rsi_1.unique_user_id = reassign_from_users_1.unique_user_id AND reassign_from_users_1.effective = 1
                LEFT JOIN sgroup reassign_from_grps_1 ON reassign_from_grps_1.unique_group_id = rsi_1.unique_group_id AND reassign_from_grps_1.effective = 1
              WHERE ria_1_1.action = 'REASSIGNED' AND ria_1_1.id = ria_1_2.id
              ORDER BY reassign_from_users_1.display_name 
            FOR XML PATH('')),1,3,'')) users,
          (STUFF((SELECT ' | ' + reassign_from_grps_1.name
              FROM review_item_action ria_1_1
                JOIN reviewer_set_instance rsi_1 ON ria_1_1.reviewer_instance_id = rsi_1.reviewer_instance_id
                LEFT JOIN suser reassign_from_users_1 ON rsi_1.unique_user_id = reassign_from_users_1.unique_user_id AND reassign_from_users_1.effective = 1
                LEFT JOIN sgroup reassign_from_grps_1 ON reassign_from_grps_1.unique_group_id = rsi_1.unique_group_id AND reassign_from_grps_1.effective = 1
              WHERE ria_1_1.action = 'REASSIGNED' AND ria_1_1.id = ria_1_2.id
              ORDER BY reassign_from_grps_1.name
            FOR XML PATH('')),1,3,'')) grps
        FROM review_item_action ria_1_2
          JOIN reviewer_set_instance rsi_1 ON ria_1_2.reviewer_instance_id = rsi_1.reviewer_instance_id
          LEFT JOIN suser reassign_from_users_1 ON rsi_1.unique_user_id = reassign_from_users_1.unique_user_id AND reassign_from_users_1.effective = 1
          LEFT JOIN sgroup reassign_from_grps_1 ON reassign_from_grps_1.unique_group_id = rsi_1.unique_group_id AND reassign_from_grps_1.effective = 1
        WHERE ria_1_2.action = 'REASSIGNED'
        GROUP BY ria_1_2.id) reassign_from_multi ON reassign_from_multi.id = ria_1.id
    WHERE ria_1.action = 'REASSIGNED') history
    LEFT JOIN review_item_action_reason reason ON history.reason_id = reason.id
    JOIN review_item item ON item.id = history.review_item_id
    LEFT JOIN spermission perm ON perm.unique_permission_id = item.unique_permission_id AND perm.effective = 1
    LEFT JOIN saccount acct ON acct.unique_account_id = item.unique_account_id AND acct.effective = 1
    LEFT JOIN business_role brole ON brole.id = item.business_role_id
    LEFT JOIN role_policy role ON role.id = item.role_id
    JOIN ( SELECT 
        ri.id AS inst_id,
        defn.id AS defn_id,
        COALESCE(staged_defn.name, defn.name) AS name,
        ri.status,
        ri.start_date,
        ri.end_date
      FROM review_definition defn
        JOIN review_instance ri ON defn.id = ri.review_definition
        LEFT JOIN review_definition staged_defn ON staged_defn.id = ri.staged_review_definition_id
      WHERE ri.status NOT IN ('PREVIEW_CANCELED')) review ON review.inst_id = history.review_instance_id

GO

CREATE VIEW review_item_reason_v AS
  SELECT 
    id,
    name,
    action_type,
    item_type,
    user_required,
    enabled
  FROM review_item_action_reason
GO

CREATE VIEW review_parameter_v AS 
  SELECT DISTINCT
    defn.name AS review_name,
    defn.unique_review_def_id
  FROM (SELECT
      review_instance.id inst_id,
      COALESCE(staged_defn.name, defn.name) AS name,
      defn.unique_review_def_id,
      defn.deleted
    FROM review_definition defn
      JOIN review_instance ON defn.id = review_instance.review_definition
      LEFT JOIN review_definition staged_defn ON staged_defn.id = review_instance.staged_review_definition_id) defn
  WHERE EXISTS (
    SELECT 1
    FROM review_instance ri
      JOIN review_item item ON item.review_instance_id = ri.id
    WHERE ri.id = defn.inst_id
    ) AND defn.deleted = 0
GO

GRANT SELECT ON reassignments_v TO ig_report_role;
GRANT SELECT ON review_item_reason_v TO ig_report_role;
GRANT SELECT ON review_parameter_v TO ig_report_role;
GO